@@ -1,3 +1,5 @@
+from typing import Never, assert_never
+
 from sonolus.backend.ops import Op
 from sonolus.script.array import Array
 from sonolus.script.array_like import ArrayLike
@@ -78,10 +80,13 @@ def _reversed(iterable):
 
 
 @meta_fn
-def _zip(*iterables):
+def _zip(*iterables, strict: bool = False):
     from sonolus.backend.visitor import compile_and_call
     from sonolus.script.containers import Pair
 
+    if validate_value(strict)._as_py_():  # type: ignore
+        raise NotImplementedError("Strict zipping is not supported")
+
     if not iterables:
         return _EmptyIterator()
 
@@ -383,6 +388,11 @@ def _super(*args):
     return super(*(arg._as_py_() if arg._is_py_() else arg for arg in args))
 
 
+@meta_fn
+def _assert_never(arg: Never, /):
+    error("Expected code to be unreachable")
+
+
 # classmethod, property, staticmethod are supported as decorators, but not within functions
 
 BUILTIN_IMPLS = {
@@ -407,6 +417,7 @@ BUILTIN_IMPLS = {
     id(reversed): _reversed,
     id(super): _super,
     id(zip): _zip,
+    id(assert_never): _assert_never,
     **MATH_BUILTIN_IMPLS,  # Includes round
     **RANDOM_BUILTIN_IMPLS,
 }
